// Copyright (C) 2016-2020
// This file is part of the Hierarchical Data Format query language (HDFql)
// For more information about HDFql, please visit the website http://www.hdfql.com

// $Rev: 234 $



#ifndef HDFQL_STRUCTURES_HEADER
	#define HDFQL_STRUCTURES_HEADER



/*! \mainpage HDFql C Structures Documentation
 *
 *  Library of HDFql C structures to manage HDF5 files
 *
 *  \n
 *  \n
 *  \li \ref SectionCursor
 *  \n
 *  \n
 *  \li \ref SectionSubcursor
 *  \n
 *  \n
 *  \li \ref SectionVariable
*/



//===========================================================
// ENABLE USAGE FROM C++
//===========================================================
#ifdef __cplusplus
	extern "C"
	{
#endif



//===========================================================
// STRUCTURES
//===========================================================
/** @defgroup SectionCursor Structures
 *  List of cursor structures.
 *  @{
*/

typedef struct
{
	int                      data_type;
	int                      count;
	int                      position;
	struct hdfql_cursor_data *head;
	struct hdfql_cursor_data *tail;
	struct hdfql_cursor_data *current;
}HDFQL_CURSOR;

struct hdfql_cursor_data
{
	void                     *data;
	int                      data_type;
	int                      size;
	struct hdfql_cursor_data *next;
	struct hdfql_cursor_data *previous;
	struct hdfql_subcursor   *subcursor;
};

/** @} */



/** @defgroup SectionSubcursor Structures
 *  List of subcursor structures.
 *  @{
*/

struct hdfql_subcursor
{
	int                         count;
	int                         position;
	struct hdfql_subcursor_data *head;
	struct hdfql_subcursor_data *tail;
	struct hdfql_subcursor_data *current;
};

struct hdfql_subcursor_data
{
	void                        *data;
	int                         size;
	struct hdfql_subcursor_data *next;
	struct hdfql_subcursor_data *previous;
};

/** @} */



/** @defgroup SectionVariable Structures
 *  List of variable structures.
 *  @{
*/

#if defined(__GNUC__) || defined(__GNUG__)   // GNU GCC/G++

	typedef struct
	{
		#if __x86_64__
			unsigned long long count;
		#else   // 32 bit
			unsigned int       count;
		#endif
		void                   *address;
	}HDFQL_VARIABLE_LENGTH;

#elif defined(_MSC_VER)   // Microsoft Visual Studio

	typedef struct
	{
		#if _WIN64
			unsigned long long count;
		#else   // 32 bit
			unsigned int       count;
		#endif
		void                   *address;
	}HDFQL_VARIABLE_LENGTH;

#else   // UNKNOWN COMPILER/ARCHITECTURE (ASSUMED TO BE 64 bit)

	typedef struct
	{
		unsigned long long count;
		void               *address;
	}HDFQL_VARIABLE_LENGTH;

#endif

/** @} */



#ifdef __cplusplus
	}
#endif



#endif   // HDFQL_STRUCTURES_HEADER

